'Raspberry Pi I/O co-processor
' Brian Millier
' Jan 2013
' Feb 10 2012 ' modified to send <CR> <LF> as a terminator to responses
' and expect a <CR> from host as a command terminator
' Done to make unit work easily with Minicom terminal, which sends <CR> and needs <CR> <LF> to display nicely

$regfile = "M328Pdef.dat"
$crystal = 18432000
$lib "i2c.lib"
$hwstack = 100
$swstack = 100
$framesize = 100
$baud = 115200

$prog &HFF , &HC7 , &HD9 , &HFF                             ' Set fuse bytes properly

Declare Function Sigmadeltaadc(chan As Byte , Gain As Byte , Rate As Byte) As Long
Declare Function Getkeyboard() As Byte
Declare Function Getkeyboard2() As Byte
Declare Sub Servoinit()
Declare Sub Setservo()
Declare Sub Mcp23s17write(regaddr As Byte , Regdata As Byte)
Declare Function Mcp23s17read(regaddr As Byte) As Byte
Declare Sub Loaddaca(dval As Word)
Declare Sub Loaddacb(dval As Word)
Declare Sub Adcroutine()
Declare Sub Dacroutine()
Declare Sub Portroutine()
Declare Sub Setadcchannel()
Declare Sub Setadcgain()
Declare Sub Setadcrate()
Declare Sub Adcread()
Declare Sub Setporta()
Declare Sub Setportb()
Declare Sub Setportamode()
Declare Sub Setportbmode()
Declare Sub Setportainputmode0()
Declare Sub Setportainputmode1()
Declare Sub Setportainputmode2()
Declare Sub Setportainputmode3()
Declare Sub Setportaoutputmode()
Declare Sub Setportbinputmode0()
Declare Sub Setportbinputmode1()
Declare Sub Setportbinputmode2()
Declare Sub Setportbinputmode3()
Declare Sub Setportboutputmode()
Declare Sub Portaoutput()
Declare Sub Portainput()
Declare Sub Portboutput()
Declare Sub Portbinput()
Declare Sub Motorabinit()
Declare Sub Motorroutine()
Declare Sub Setmotora()
Declare Sub Setmotorb()
Declare Sub Motoraspeed()
Declare Sub Motoraforward()
Declare Sub Motorareverse()
Declare Sub Motorabrake()
Declare Sub Motorbspeed()
Declare Sub Motorbforward()
Declare Sub Motorbreverse()
Declare Sub Motorbbrake()
Declare Sub Servoroutine()
Declare Sub Servoset()
Declare Sub Encoderroutine()
Declare Sub Getencodervalue()
Declare Sub Getencodermoved()
Declare Sub Encoderzero()
Declare Sub Stepcw()
Declare Sub Stepccw()
Declare Sub Stepperroutine()
Declare Sub Stepmove()
Declare Sub Steprate()
Declare Sub Stepnumsteps()
Declare Sub Setmicrel()
Declare Sub Initkeyboard()
Declare Sub Keyboardroutine()
Declare Sub Readkeyboard()
Declare Sub Readkeyboardnw()
Declare Sub Nokeyspressed()
Declare Sub Keyboardremove()

Dim Lsb As Byte
Dim Msb As Byte
Dim I As Byte
Dim A As Byte
Dim B As Byte
Dim Adcarray(4) As Byte
Dim Adcval As Long
Dim Tlong As Long
Dim At1 As Byte At Adcval Overlay
Dim At2 As Byte At Adcval + 1 Overlay
Dim At3 As Byte At Adcval + 2 Overlay
Dim At4 As Byte At Adcval + 3 Overlay
Dim Configbyte As Byte
Dim Adcgain As Byte
Dim Adcchan As Byte
Dim Adcrate As Byte
Dim Adcbusy As Byte
Dim Adctemp As Byte
Dim Retemp As Boolean
Dim Encodermoved As Boolean
Dim Encodervalue As Integer
Dim W As Word
Dim Stemp As Word
Dim Tmp1 As Byte
Dim S As String * 1
Dim T As String * 10
Dim Cmderr As Byte
Dim Ftemp As Single
Dim Phase(8) As Byte
Dim P As Byte
Dim Phs As Byte
Dim Position As Integer
Dim Numsteps As Integer
Dim Srate As Integer
Dim Stepaccel As Byte

' Definition of MCP23S17 I/O Expander Port Base registers
Dim Mcp23s17wr As Byte
Dim Mcp23s17rd As Byte
Mcp23s17wr = &H40
Mcp23s17rd = &H41

' Following Constants apply when IOCON.BANK bit = 0 !!!
Const Mcp23s17iodira = 0
Const Mcp23s17iopola = 2
Const Mcp23s17gpintena = 4
Const Mcp23s17defvala = 6
Const Mcp23s17intcona = 8
Const Mcp23s17iocon = 10
Const Mcp23s17gppua = 12
Const Mcp23s17intfa = 14
Const Mcp23s17intcapa = 16
Const Mcp23s17gpioa = 18
Const Mcp23s17olata = 20

Const Mcp23s17iodirb = 1
Const Mcp23s17iopolb = 3
Const Mcp23s17gpintenb = 5
Const Mcp23s17defvalb = 7
Const Mcp23s17intconb = 9
Const Mcp23s17gppub = 13
Const Mcp23s17intfb = 15
Const Mcp23s17intcapb = 17
Const Mcp23s17gpiob = 19
Const Mcp23s17olatb = 21


Const True = 1
Const False = 0

Const Id = "Raspberry Pi I/O board" + Chr(13) + Chr(10)

'Stepper motor  half-step sequence
Const Step1 = &B11001100
Const Step2 = &B11111100
Const Step3 = &B00111100
Const Step4 = &B00111111
Const Step5 = &B00110011
Const Step6 = &B11110011
Const Step7 = &B11000011
Const Step8 = &B11001111

' set up the stepper motor phase array from above constants
Phase(1) = Step1
Phase(2) = Step2
Phase(3) = Step3
Phase(4) = Step4
Phase(5) = Step5
Phase(6) = Step6
Phase(7) = Step7
Phase(8) = Step8



' Definition of MCP3424 I2C Base Address
Const Mcp3424address = &HD0

Config Serialin = Buffered , Size = 40
Clear Serialin

Echo Off                                                    ' stop INPUT routine from echoing characters

Config Sda = Portc.4

Config Scl = Portc.5
I2cinit

' Micrel MIC5821 Power latch control pin initialization
Config Portd.2 = Output                                     ' Micrel Mic5891 , 5821  Strobe Pins
Reset Portd.2
Micrelstrobe Alias Portd.2

' MCP23S17 16-bit I/O Expander Control Pin initialization
Config Portb.2 = Output                                     ' SPI port -SS pin connected to MCP23S17 -CS pin
Set Portb.2
Mcp23s17ncs Alias Portb.2

' MCP4822 dual 12 bit DAC control pin initialization
Config Portd.7 = Output
Set Portd.7
Mcp4822ncs Alias Portd.7

Config Spi = Hard , Interrupt = Off , Data Order = Msb , Master = Yes , Polarity = Low , Phase = 0 , Clockrate = 16 , Noss = 1
Spiinit

 ' configure the port pins  for BOURNS Rotary encoder (Must be  Bourns ECW1D-B24-BC0024 or equiv)
 ' Rotary encoder is on ports D.4 and D.4
Config Portd.3 = Input
Set Portd.3                                                 ' make pullup active
Config Portd.4 = Input
Set Portd.4                                                 ' make pullup active
Eicra = &B00001000                                          ' falling edge interrupt on INT1 (Rotary encoder pin)
' setup the interrupt ISR that handles the rotary encoder
On Int1 Reisr:
Enable Int1
Enable Interrupts

' Define the L298 Motor Driver port pins
L298in1 Alias Portc.0
L298in2 Alias Portc.1
L298in3 Alias Portc.2
L298in4 Alias Portc.3
L298ena Alias Portd.6
L298enb Alias Portd.5

Call Motorabinit()

Srate = 1000
                                             ' 1000 us time per stepper move as default
'******************
' Command Loop    *
'******************

Do
 Cmderr = 0
 Do
  Inputbin S
 Loop Until S <> Chr(10)                                    'gobble up <LF> sent after each command ( for those commands that have no numeric param)
   Select Case S
    Case Is = "A" : Call Adcroutine()
    Case Is = "B" : Call Stepperroutine()
    Case Is = "C" : Call Setmicrel()
    Case Is = "D" : Call Dacroutine()
    Case Is = "E" : Call Encoderroutine()
    Case Is = "K" : Call Keyboardroutine()
    Case Is = "M" : Call Motorroutine()
    Case Is = "P" : Call Portroutine()
    Case Is = "S" : Call Servoroutine()
    Case Is = "?" : Print Id;
    Case Else : Cmderr = 99
   End Select
   If Cmderr = 0 Then
    Print "OK";
   Else
    Print "ER";
    Print Cmderr;
   End If
   Print Chr(13);                                           '<CR> for terminal use
   Print Chr(10);                                           ' LF is the default terminator for Python Py-serial routines
Loop

Sub Setmicrel()
 Input T
 Ftemp = Val(t)
 If Ftemp < 0 Then Cmderr = 16
 If Ftemp > 255 Then Cmderr = 16
 If Cmderr = 0 Then
   I = Ftemp
   Spiout I , 1
   Set Micrelstrobe
   Waitus 1
   Reset Micrelstrobe
 End If
End Sub

Sub Keyboardroutine()
   Inputbin S
   Select Case S
    Case Is = "I" : Call Initkeyboard()
    Case Is = "G" : Call Readkeyboard()
    Case Is = "F" : Call Readkeyboardnw()
    Case Is = "N" : Call Nokeyspressed()
    Case Is = "R" : Call Keyboardremove()
    Case Else Cmderr = 17
   End Select
End Sub

' Return MCP23S17 PortA to input mode 0, ( used after keypad routines used)
Sub Keyboardremove()
  Call Setportainputmode0()
End Sub

' scan keypad until a key pressed, and return it
Sub Readkeyboard()
 I = Getkeyboard()
 Print I;
 Print Chr(13);                                             '<CR> for terminal use
 Print Chr(10);
End Sub

' scan keypad once - return key pressed (1-16) or a zero if no key pressed
Sub Readkeyboardnw()
 I = Getkeyboard2()
 Print I;
 Print Chr(13);                                             '<CR> for terminal use
 Print Chr(10);
End Sub

' Set up MCP23S17 PORTA for 4X4 Keypad scanning
Sub Initkeyboard()
' set port A0- A3 for input, pulled up , PORT A4-7 as output
 A = Mcp23s17iodira
 B = 15
 Call Mcp23s17write(a , B)
' set PORT A for polarity change on PA0- PA3
 A = Mcp23s17iopola
 Call Mcp23s17write(a , B)
' set PORT A0- A3 for  pullup
 A = Mcp23s17gppua
 Call Mcp23s17write(a , B)
End Sub

' Scans keypad continuously until a key is pressed
' Return key number 1-16
' Blocking!
Function Getkeyboard() As Byte
 Local Row As Byte
 Local Col As Byte
 Local I As Byte
 Local Tmp4 As Byte
 Local Flg As Byte
 Local Key As Byte
 Local A2 As Byte
 Local B2 As Byte
 Do
  Tmp4 = 12
  Row = 127
  Flg = 0
  Key = 0
  For I = 1 To 4
    B2 = Row
    A2 = Mcp23s17olata
    Call Mcp23s17write(a2 , B2)
    Waitms 1
    'read column
    A2 = Mcp23s17gpioa
    Col = Mcp23s17read(a2)
    Col = Col And 15
    If Col > 0 Then
     Flg = 1
     Waitms 25
     A2 = Mcp23s17gpioa
     Col = Mcp23s17read(a2)
     Col = Col And 15
     If Col = 0 Then Flg = 0
    End If
    If Col = 1 Then Key = Tmp4
    If Col = 2 Then Key = Tmp4 + 1
    If Col = 4 Then Key = Tmp4 + 2
    If Col = 8 Then Key = Tmp4 + 3
    If Flg = 1 Then Exit For
   Shift Row , Right , 1
   Row = Row Or 128
   Tmp4 = Tmp4 - 4
  Next I
 Loop Until Flg = 1
 Getkeyboard = Key + 1
End Function

' Non-Blocking Keyboard Scan- return key pressed (1-16)  or a Zero, if no key pressed
Function Getkeyboard2() As Byte
 Local Row As Byte
 Local Col As Byte
 Local I As Byte
 Local Tmp4 As Byte
 Local Flg As Byte
 Local Key As Byte
 Local A2 As Byte
 Local B2 As Byte
  Tmp4 = 12
  Row = 127
  Flg = 0
  Key = 0
  For I = 1 To 4
    B2 = Row
    A2 = Mcp23s17olata
    Call Mcp23s17write(a2 , B2)
    Waitms 1
    'read column
    A2 = Mcp23s17gpioa
    Col = Mcp23s17read(a2)
    Col = Col And 15
    If Col > 0 Then
     Flg = 1
     Waitms 25
     A2 = Mcp23s17gpioa
     Col = Mcp23s17read(a2)
     Col = Col And 15
     If Col = 0 Then Flg = 0
    End If
    If Col = 1 Then Key = Tmp4
    If Col = 2 Then Key = Tmp4 + 1
    If Col = 4 Then Key = Tmp4 + 2
    If Col = 8 Then Key = Tmp4 + 3
    If Flg = 1 Then Exit For
   Shift Row , Right , 1
   Row = Row Or 128
   Tmp4 = Tmp4 - 4
  Next I
 If Flg = 1 Then
   Getkeyboard2 = Key + 1
 Else
   Getkeyboard2 = 0
 End If
End Function

' Wait until no keypad keys pressed
' Blocking!
Sub Nokeyspressed()
    Local A2 As Byte
    Local B2 As Byte
    Local Col As Byte
    ' lower all row lines
    B2 = 15
    A2 = Mcp23s17olata
    Call Mcp23s17write(a2 , B2)
    Waitms 1
    Do
     'read columns
     A2 = Mcp23s17gpioa
     Col = Mcp23s17read(a2)
     Col = Col And 15
     Waitms 10
    Loop Until Col = 0
End Sub

Sub Stepperroutine()
  Inputbin S
   Select Case S
     Case Is = "F" : Call Stepcw()
     Case Is = "R" : Call Stepccw()
     Case Is = "M" : Call Stepmove()
     Case Is = "T" : Call Steprate()
     Case Is = "S" : Call Stepnumsteps()
     Case Else Cmderr = 13
   End Select
End Sub

Sub Stepnumsteps()
 Input T
 Ftemp = Val(t)
 If Ftemp < -32768 Then Cmderr = 14
 If Ftemp > 32767 Then Cmderr = 14
 If Cmderr = 0 Then
  Numsteps = Ftemp
 End If
End Sub

Sub Steprate()
 Input T
 Ftemp = Val(t)
 If Ftemp < -32768 Then Cmderr = 15
 If Ftemp > 32767 Then Cmderr = 15
 If Cmderr = 0 Then
  Srate = Ftemp
 End If
End Sub

Sub Stepmove()
  Position = 0
  Stepaccel = 10
  Do
   If Numsteps > 0 Then
    Call Stepcw()
   Else
    Call Stepccw()
   End If
   For I = 1 To Stepaccel
    Waitus Srate
   Next
   Decr Stepaccel
   If Stepaccel < 1 Then Stepaccel = 1
  Loop Until Position = Numsteps
End Sub

Sub Encoderroutine()
  Inputbin S
   Select Case S
     Case Is = "V" : Call Getencodervalue()
     Case Is = "M" : Call Getencodermoved()
     Case Is = "Z" : Call Encoderzero()
     Case Else Cmderr = 11
   End Select
End Sub

Sub Getencodervalue()
 Print Encodervalue;
 Print Chr(13);                                             '<CR> for terminal use
 Print Chr(10);
 Encodermoved = False
End Sub

Sub Getencodermoved()
 Print Encodermoved;
 Print Chr(13);                                             '<CR> for terminal use
 Print Chr(10);
End Sub

Sub Encoderzero()
 Encodervalue = 0
 Encodermoved = False
End Sub

Sub Servoroutine()
  Inputbin S
   Select Case S
     Case Is = "S" : Call Setservo()
     Case Is = "I" : Call Servoinit()
     Case Else Cmderr = 10
   End Select
End Sub

Sub Setservo()
 Input T
 Ftemp = Val(t)                                             ' get the floating point servo value in the range of 0.0 - 100.0 percent
 Ftemp = Ftemp * 23
 If Ftemp < 0 Then Ftemp = 0
 If Ftemp > 2304 Then Ftemp = 2304
 Stemp = Int(ftemp)                                         '
 Stemp = Stemp + 2304
 Compare1a = Stemp
End Sub

Sub Motorroutine()
  Inputbin S
   Select Case S
     Case Is = "A" : Call Setmotora()
     Case Is = "B" : Call Setmotorb()
     Case Else Cmderr = 8
   End Select
End Sub

Sub Setmotora()
  Inputbin S
   Select Case S
     Case Is = "F" : Call Motoraforward()
     Case Is = "R" : Call Motorareverse()
     Case Is = "S" : Call Motoraspeed()
     Case Is = "B" : Call Motorabrake()
     Case Else Cmderr = 9
   End Select
End Sub

Sub Motoraforward()
 Set L298in1
 Reset L298in2
End Sub

Sub Motorareverse()
 Reset L298in1
 Set L298in2
End Sub

Sub Motoraspeed()
 Input T
 B = Val(t)
 Ocr0a = B
End Sub

Sub Motorabrake()
 Reset L298in1
 Reset L298in2
End Sub

Sub Setmotorb()
  Inputbin S
   Select Case S
     Case Is = "F" : Call Motorbforward()
     Case Is = "R" : Call Motorbreverse()
     Case Is = "S" : Call Motorbspeed()
     Case Is = "B" : Call Motorbbrake()
     Case Else Cmderr = 9
   End Select
End Sub

Sub Motorbforward()
 Set L298in3
 Reset L298in4
End Sub

Sub Motorbreverse()
 Reset L298in3
 Set L298in4
End Sub

Sub Motorbspeed()
 Input T
 B = Val(t)
 Ocr0b = B
End Sub

Sub Motorbbrake()
 Reset L298in3
 Reset L298in4
End Sub

Sub Portroutine()
  Inputbin S
   Select Case S
     Case Is = "A" : Call Setporta()
     Case Is = "B" : Call Setportb()
     Case Else Cmderr = 5
   End Select
End Sub

Sub Setporta()
  Inputbin S
   Select Case S
     Case Is = "M" : Call Setportamode()
     Case Is = "O" : Call Portaoutput()
     Case Is = "I" : Call Portainput()
     Case Else Cmderr = 6
   End Select
End Sub

Sub Setportb()
  Inputbin S
   Select Case S
     Case Is = "M" : Call Setportbmode()
     Case Is = "O" : Call Portboutput()
     Case Is = "I" : Call Portbinput()
     Case Else Cmderr = 6
   End Select
End Sub

'*******************
'*     PORT A      *
'*******************

Sub Setportamode()
 Input I
 If I > 4 Then
  Cmderr = 7
 Else
 Select Case I
  Case Is = 0 : Call Setportainputmode0()
  Case Is = 1 : Call Setportainputmode1()
  Case Is = 2 : Call Setportainputmode2()
  Case Is = 3 : Call Setportainputmode3()
  Case Is = 4 : Call Setportaoutputmode()
 End Select
 End If :
End Sub

Sub Setportainputmode0()
' set port A for input
 A = Mcp23s17iodira
 B = 255
 Call Mcp23s17write(a , B)
' set PORT A for no polarity change on input pins
 A = Mcp23s17iopola
 B = 0
 Call Mcp23s17write(a , B)
' set PORT A for no pullup
 A = Mcp23s17gppua
 B = 0
 Call Mcp23s17write(a , B)
End Sub

Sub Setportainputmode1()
' set port A for input
 A = Mcp23s17iodira
 B = 255
 Call Mcp23s17write(a , B)
' set PORT A for polarity change on input pins
 A = Mcp23s17iopola
 B = 255
 Call Mcp23s17write(a , B)
' set PORT A for no pullup
 A = Mcp23s17gppua
 B = 0
 Call Mcp23s17write(a , B)
End Sub

Sub Setportainputmode2()
' set port A for input
 A = Mcp23s17iodira
 B = 255
 Call Mcp23s17write(a , B)
' set PORT A for no polarity change on input pins
 A = Mcp23s17iopola
 B = 0
 Call Mcp23s17write(a , B)
' set PORT A for  pullup
 A = Mcp23s17gppua
 B = 255
 Call Mcp23s17write(a , B)
End Sub

Sub Setportainputmode3()
' set port A for input
 A = Mcp23s17iodira
 B = 255
 Call Mcp23s17write(a , B)
' set PORT A for polarity change on input pins
 A = Mcp23s17iopola
 B = 255
 Call Mcp23s17write(a , B)

' set PORT A for  pullup
 A = Mcp23s17gppua
 B = 255
 Call Mcp23s17write(a , B)
End Sub


Sub Setportaoutputmode()
' set port A for output
 A = Mcp23s17iodira
 B = 0
 Call Mcp23s17write(a , B)
End Sub

Sub Portaoutput()
 Input T
 B = Val(t)
 A = Mcp23s17olata
Call Mcp23s17write(a , B)
End Sub

Sub Portainput()
 A = Mcp23s17gpioa
 I = Mcp23s17read(a)
 Print I;
 Print Chr(13);                                             '<CR> for terminal use
 Print Chr(10);
End Sub

'**************
'   PORT B    *
'**************

Sub Setportbmode()
 Input I
 If I > 4 Then
  Cmderr = 7
 Else
 Select Case I
  Case Is = 0 : Call Setportbinputmode0()
  Case Is = 1 : Call Setportbinputmode1()
  Case Is = 2 : Call Setportbinputmode2()
  Case Is = 3 : Call Setportbinputmode3()
  Case Is = 4 : Call Setportboutputmode()
 End Select
 End If :
End Sub

Sub Setportbinputmode0()
' set port B for input
 A = Mcp23s17iodirb
 B = 255
 Call Mcp23s17write(a , B)
' set PORTB for no polarity change on input pins
 A = Mcp23s17iopolb
 B = 0
 Call Mcp23s17write(a , B)
' set PORTB for no pullup
 A = Mcp23s17gppub
 B = 0
 Call Mcp23s17write(a , B)
End Sub

Sub Setportbinputmode1()
' set port B for input
 A = Mcp23s17iodirb
 B = 255
 Call Mcp23s17write(a , B)
' set PORT B for polarity change on input pins
 A = Mcp23s17iopolb
 B = 255
 Call Mcp23s17write(a , B)
' set PORT B for no pullup
 A = Mcp23s17gppub
 B = 0
 Call Mcp23s17write(a , B)
End Sub

Sub Setportbinputmode2()
' set port B for input
 A = Mcp23s17iodirb
 B = 255
 Call Mcp23s17write(a , B)
' set PORT B for no polarity change on input pins
 A = Mcp23s17iopolb
 B = 0
 Call Mcp23s17write(a , B)
' set PORT B for  pullup
 A = Mcp23s17gppub
 B = 255
 Call Mcp23s17write(a , B)
End Sub

Sub Setportbinputmode3()
' set port B for input
 A = Mcp23s17iodirb
 B = 255
 Call Mcp23s17write(a , B)
' set PORT B for polarity change on input pins
 A = Mcp23s17iopolb
 B = 255
 Call Mcp23s17write(a , B)

' set PORT B for  pullup
 A = Mcp23s17gppub
 B = 255
 Call Mcp23s17write(a , B)
End Sub

Sub Setportboutputmode()
' set port B for output
 A = Mcp23s17iodirb
 B = 0
 Call Mcp23s17write(a , B)
End Sub

Sub Portboutput()
 Input T
 B = Val(t)
 A = Mcp23s17olatb
Call Mcp23s17write(a , B)
End Sub

Sub Portbinput()
 A = Mcp23s17gpiob
 I = Mcp23s17read(a)
 Print I;
 Print Chr(13);                                             '<CR> for terminal use
 Print Chr(10);
End Sub

'*****************************************END PORT B *********************************************************************

Sub Adcroutine()
  Inputbin S
   Select Case S
    Case Is = "C" : Call Setadcchannel()
    Case Is = "G" : Call Setadcgain()
    Case Is = "R" : Call Setadcrate()
    Case Is = "I" : Call Adcread()
   End Select
End Sub

Sub Setadcchannel()
  Input I
  If I > 3 Then
    Cmderr = 1
  Else
   Adcchan = I
  End If
  End Sub

Sub Setadcgain()
 Input I
  If I > 3 Then
    Cmderr = 2
  Else
   Adcgain = I
  End If
End Sub

Sub Setadcrate()
 Input I
  If I > 3 Then
    Cmderr = 3
  Else
   Adcrate = I
  End If
End Sub

Sub Adcread()
 Tlong = Sigmadeltaadc(adcchan , Adcgain , Adcrate)
 Print Tlong;
 Print Chr(13);                                             '<CR> for terminal use
 Print Chr(10);
End Sub

Sub Dacroutine()
 Cmderr = 4
 Inputbin S
 If S = "A" Then Cmderr = 0
 If S = "B" Then Cmderr = 0
 Input W
 If W < 0 Then Cmderr = 4
 If W > 4095 Then Cmderr = 4
 If Cmderr = 0 Then
  If S = "A" Then Call Loaddaca(w)
  If S = "B" Then Call Loaddacb(w)
 End If
End Sub

Sub Loaddaca(dval As Word)
 Local Daca As Word
  Daca = Dval Or &H3000
  Msb = High(daca)
  Lsb = Low(daca)
  Reset Mcp4822ncs
  Spiout Msb , 1
  Spiout Lsb , 1
  Set Mcp4822ncs
End Sub

Sub Loaddacb(dval As Word)
 Local Dacb As Word
  Dacb = Dval Or &HB000
  Msb = High(dacb)
  Lsb = Low(dacb)
  Reset Mcp4822ncs
  Spiout Msb , 1
  Spiout Lsb , 1
  Set Mcp4822ncs
End Sub

Sub Mcp23s17write(regaddr As Byte , Regdata As Byte)
 Reset Mcp23s17ncs
 Spiout Mcp23s17wr , 1
 Spiout Regaddr , 1
 Spiout Regdata , 1
 Set Mcp23s17ncs
End Sub

Function Mcp23s17read(regaddr As Byte) As Byte
 Local T As Byte
 Reset Mcp23s17ncs
 Spiout Mcp23s17rd , 1
 Spiout Regaddr , 1
 Spiin T , 1
 Set Mcp23s17ncs
 Mcp23s17read = T
End Function

Function Sigmadeltaadc(chan As Byte , Gain As Byte , Rate As Byte) As Long
  Adctemp = Chan
  Shift Adctemp , Left , 5
  Configbyte = Adctemp
  Configbyte = Configbyte + Gain
  Adctemp = Rate
  Shift Adctemp , Left , 2
  Configbyte = Configbyte + Adctemp
  Configbyte = Configbyte Or 128                            ' add the start conversion bit
  I2csend Mcp3424address , Configbyte

 If Rate < 3 Then                                           ' for 12,14,16 bit conversions, the MCP3424 returns 2 data bytes +config
 Do
  I2creceive Mcp3424address , Adcarray(1) , 0 , 3
   Adcbusy = Adcarray(3) And 128
   At1 = Adcarray(2)
   At2 = Adcarray(1)
   If At2 > 127 Then                                        'sign extend bytes at3,at4, which overlay the MS byte of the long var Adcval
    At3 = 255
    At4 = 255
   Else
    At3 = 0
    At4 = 0
   End If
 Loop Until Adcbusy = 0

 Else                                                       ' for 18 bit conversions, MCP3424 returns 3 data bytes + config register
  Do
   I2creceive Mcp3424address , Adcarray(1) , 0 , 4
   Adcbusy = Adcarray(4) And 128
   At1 = Adcarray(3)
   At2 = Adcarray(2)
   At3 = Adcarray(1)
   If At3 > 127 Then                                        ' sign extend at4, which overlays the MS byte of long var adcval
    At4 = 255
   Else
    At4 = 0
   End If
  End If
  Loop Until Adcbusy = 0
  Sigmadeltaadc = Adcval
End Function

 ' setup Servo output on OC1A
 ' use Timer Mode 10 PWM , phase correct
 ' set output pulse = 1.5 ms for servo center position

Sub Servoinit()
 Tccr1a = &B10000010
 Tccr1b = &B00010010
 Icr1 = 23040
 Compare1a = 3456
 Config Portb.1 = Output
End Sub

Sub Motorabinit()
 Tccr0a = &B10100001
 Tccr0b = &B00000011                                        ' use MCU CLK / 64  (563 HZ PWM)
 Ocr0a = 240
 Ocr0b = 240
' configure the PWM output pins
 Config L298ena = Output
 Config L298enb = Output
 Reset L298in1
 Reset L298in2
 Reset L298in3
 Reset L298in4
' configure the Direction lines
 Config L298in1 = Output
 Config L298in2 = Output
 Config L298in3 = Output
 Config L298in4 = Output
End Sub

Sub Stepccw()
  Decr Position
  P = Position Mod 8
  Incr P
  Phs = Phase(p)
  Spiout Phs , 1
  Set Micrelstrobe
  Waitus 1
  Reset Micrelstrobe
  Waitms 1
End Sub

Sub Stepcw()
  Incr Position
  P = Position Mod 8
  Incr P
  Phs = Phase(p)
  Spiout Phs , 1
  Set Micrelstrobe
  Waitus 1
  Reset Micrelstrobe
  Waitms 1
End Sub

' This is the ISR that handles the Rotary Encoder movement
Reisr:
 Retemp = Pind.4
 Encodermoved = 1
  If Retemp = 0 Then
   Incr Encodervalue
  Else
   Decr Encodervalue
  End If
 Bitwait Pind.3 , Set
Return

End